// Client.h : Declaration of the CClient

#ifndef __CLIENT_H_
#define __CLIENT_H_

#include "resource.h"       // main symbols
#include "EventHandler.h"

/////////////////////////////////////////////////////////////////////////////
// Helper Macros

#define CHECKHR(hr, desc) { if (FAILED(hr)) { Error(desc,0,0,GUID_NULL,hr); return E_FAIL; } }
#define CHECKHR_UNLOCK(hr, desc) { if (FAILED(hr)) { Error(desc,0,0,GUID_NULL,hr); Unlock(); return E_FAIL; } }
#define CHECKRESULTS_UNLOCK(hr, success, desc) { if (FAILED(hr) || !success) { Error(desc,0,0,GUID_NULL,hr); Unlock(); return E_FAIL; } }


#define RETURN_ERROR(desc) { hr = E_FAIL; 	hr = Error(desc, IID_IClient, hr); 	return hr; 	}
#define RETURN_ERROR_INSIDE(desc) { *bstrOutError	= desc; 	return false; 	}


#define MACROBEGINTRY _bstr_t			bstrErrDescription(L"");		\
	HRESULT			hr = S_OK;											\
	try																	\
	{	
#define MACROENDTRY }													\
	catch(_com_error err)												\
	{																	\
		bstrErrDescription = err.Description();							\
		hr = err.Error();												\
	}																	\
	catch(...)															\
	{																	\
	bstrErrDescription = L"Some unnatended Error";						\
		hr = E_FAIL;													\
	}																	\
	if(FAILED(hr))														\
		hr = Error((LPOLESTR)bstrErrDescription, IID_IClient, hr);	\
	return hr;		

#define MACROBEGINTRY_INSIDE _bstr_t			bstrErrDescription(L"");\
	HRESULT			hr = S_OK;											\
	try																	\
	{	
#define MACROENDTRY_INSIDE }											\
	catch(_com_error err)												\
	{																	\
		bstrErrDescription = err.Description();							\
		hr = err.Error();												\
	}																	\
	catch(...)															\
	{																	\
		bstrErrDescription = L"Some unnatended Error";					\
		hr = E_FAIL;													\
	}																	\
	*bstrOutError	= bstrErrDescription;								\
	if(FAILED(hr))														\
		return false;													\
	else																\
		return true;

/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
struct SCarry
	{
		LPVOID		pClass;
		BSTR		bstrName;
		BSTR		bstrValue;
	};
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// CClient
class ATL_NO_VTABLE CClient : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CClient, &CLSID_Client>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CClient>,
	public IDispatchImpl<IClient, &IID_IClient, &LIBID_REPLICATIONCLIENTLib>
{
public:
	CClient();

DECLARE_REGISTRY_RESOURCEID(IDR_CLIENT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

DECLARE_CLASSFACTORY_SINGLETON(CClient);

BEGIN_COM_MAP(CClient)
	COM_INTERFACE_ENTRY(IClient)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(CClient)
END_CONNECTION_POINT_MAP()


// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IClient
public:

public:
	
	HRESULT							FinalConstruct();
	HRESULT							FinalRelease();
	bool							ComPing();
	bool							Readvise();
	bool							Unadvise();

	void							ReadviseAllLauncher();
	void							ReadviseAll();

	void							LookupSignals();
	_bstr_t							GetServerName();
	_bstr_t							GetClientName();

	void							Tell2Server(LPTSTR szPathLocal, LPTSTR szPathServer);

private:

	void							MyLock();
	void							MyUnlock();

	bool							ReadSignals();
	void							StartLookupSignals();
	void							StopLookupSignals();
	bool							LaunchObserver();
	
	

	CEventHandler					*m_pHandler;
	CComPtr<IUnknown>				m_pUnk;
	CRITICAL_SECTION				m_cs;
	
	
	TCHAR							m_szServerName[512];
	TCHAR							m_szClientName[512];
	TCHAR							m_szMessage[512];
	

	HANDLE  						m_evtStopThread;
	HANDLE							m_hThread;
	DWORD							m_ThreadId;
	bool							m_bContinueToRun;
	bool							m_bRunningFirstTime;
};

#endif //__CLIENT_H_
