#ifndef __SHAREMANNAGER_H__
#define __SHAREMANNAGER_H__

#define SID_SIZE 96

bool	GetAccess( LPTSTR AccountName, SECURITY_DESCRIPTOR &sd, LPTSTR &szError )
{
	DWORD			cbSid;
	PSID			pSid					= NULL;
	WCHAR			RefDomain[DNLEN + 1];
	DWORD			cchDomain				= DNLEN + 1;
	SID_NAME_USE	peUse;
	DWORD			dwAclSize;
	PACL			pDacl					= NULL;
	
	// initial allocation attempt for Sid
    cbSid		= SID_SIZE;
	
    pSid		= (PSID)HeapAlloc(GetProcessHeap(), 0, cbSid);
    if(pSid == NULL) 
	{
		swprintf(szError, L"Error in GetAccess:  %s", "HeapAlloc error.");
        return false;
    }


	
	// get the Sid associated with the supplied user/group name
    if(!LookupAccountName(
							NULL,       // default lookup logic
							AccountName,// user/group of interest from commandline
							pSid,       // Sid buffer
							&cbSid,     // size of Sid
							RefDomain,  // Domain account found on (unused)
							&cchDomain, // size of domain in chars
							&peUse
						)) 
	{
		LPVOID lpMsgBuf;
		FormatMessage( 
						FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM | 
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
						(LPTSTR) &lpMsgBuf,
						0,
						NULL 
						);
		// Display the string.
		wsprintf(szError, L"Error in LookupAccountName:  %s", (LPCTSTR)lpMsgBuf);
		// Free the buffer.
		LocalFree( lpMsgBuf );
		
		return false;
	}

	// compute size of new acl
    dwAclSize		=	sizeof(ACL) +
						1 * ( sizeof(ACCESS_ALLOWED_ACE) - sizeof(DWORD) ) +
						GetLengthSid(pSid) ;

	// allocate storage for Acl
    pDacl			= (PACL)HeapAlloc(GetProcessHeap(), 0, dwAclSize);
    if(pDacl == NULL)
	{
		wsprintf(szError, L"Error in GetAccess: HeapAlloc.");
		goto cleanup;
	}

    if(!InitializeAcl(pDacl, dwAclSize, ACL_REVISION))
	{
		wsprintf(szError, L"Error in GetAccess: InitializeAcl.");
		goto cleanup;
	}


	// grant GENERIC_ALL (Full Control) access
    if(!AddAccessAllowedAce(
							pDacl,
							ACL_REVISION,
							GENERIC_ALL,
							pSid
							))													
	{
		wsprintf(szError, L"Error in GetAccess: AddAccessAllowedAce.");
		goto cleanup;
	}

    if(!InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION))        
	{
		wsprintf(szError, L"Error in GetAccess: InitializeSecurityDescriptor.");
		goto cleanup;
	}

    if(!SetSecurityDescriptorDacl(&sd, TRUE, pDacl, FALSE)) 
	{
		LPVOID lpMsgBuf;
		FormatMessage( 
						FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM | 
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						GetLastError(),
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
						(LPTSTR) &lpMsgBuf,
						0,
						NULL 
						);
		// Display the string.
		wsprintf(szError, L"Error in SetSecurityDescriptorDacl:  %s", (LPCTSTR)lpMsgBuf);
		// Free the buffer.
		LocalFree( lpMsgBuf );

        goto cleanup;
    }


	return true;

cleanup:
    // free allocated resources
    if(pDacl != NULL)
        HeapFree(GetProcessHeap(), 0, pDacl);

    if(pSid != NULL)
        HeapFree(GetProcessHeap(), 0, pSid);

	return false;
}

bool	ShareCreate( LPTSTR pszDirectoryToShare, LPTSTR pszShareName, SECURITY_DESCRIPTOR sd, LPTSTR &szError )
{
	

	SHARE_INFO_502				si502;
    NET_API_STATUS				nas;
	
	si502.shi502_netname				= (LPTSTR) pszShareName;
    si502.shi502_type					= STYPE_DISKTREE;			// disk drive
    si502.shi502_remark					= L"Share 4 replication";
    si502.shi502_permissions			= 0;
    si502.shi502_max_uses				= SHI_USES_UNLIMITED;
    si502.shi502_current_uses			= 0;
    si502.shi502_path					= (LPTSTR) pszDirectoryToShare;
    si502.shi502_passwd					= NULL;
    si502.shi502_reserved				= 0;
    si502.shi502_security_descriptor	= &sd;

    nas = NetShareAdd(
						(LPTSTR) NULL,        // share is on local machine
						502,					// info-level
						(LPBYTE)&si502,			// info-buffer
						NULL					// don't bother with parm
						);

    if(nas != NO_ERROR) 
	{
		//isnt an error		
		//or to delete the share and create it again?
		if ( nas!= NERR_DuplicateShare)			
		{
			TCHAR		szMsgBuf[256];
			
			switch (nas)
			{
			case	ERROR_ACCESS_DENIED:	wcscpy(szMsgBuf, L"The user does not have access to the requested information.");break;
			case	ERROR_INVALID_LEVEL:	wcscpy(szMsgBuf, L"The value specified for the level parameter is invalid.");break;
			case	ERROR_INVALID_NAME:		wcscpy(szMsgBuf, L"The character or file system name is invalid.");break;
			case	ERROR_INVALID_PARAMETER:wcscpy(szMsgBuf, L"The specified parameter is invalid.");break;
			case	NERR_DuplicateShare:	wcscpy(szMsgBuf, L"The share name is already in use on this server.");break;
			case	NERR_RedirectedPath:	wcscpy(szMsgBuf, L"The operation is invalid for a redirected resource. The specified device name is assigned to a shared resource.");break;
			case	NERR_UnknownDevDir:		wcscpy(szMsgBuf, L"The device or directory does not exist.");break;
				
			default:						wcscpy(szMsgBuf, L"Unspecified error.");break;
			}
			
			// Display the string.
			swprintf(szError, L"Error in NetShareAdd:  %s", szMsgBuf);
			
			return false;
		}
    }

	return true;
}

//*********************************************************************************************
//LookupShare 
//management of share
//
//*********************************************************************************************
bool LookupShare(LPTSTR  pszPathRoot, LPTSTR pszDirectory, LPTSTR AccountName, LPTSTR &szError)
{
	SECURITY_DESCRIPTOR			sd;

	TCHAR				pszSharePath[1024]		= {0};
	TCHAR				pszShareName[1024]		= {0};

	if ( ( wcslen(pszPathRoot) <=4 ) || ( wcslen(pszDirectory) <=2 ) )
	{
		swprintf(szError, L"Error in LookupShare:  Too small paths.");
		return false;
	}

	if	( pszDirectory[(wcslen(pszDirectory)-1)] == '\\' )
	{
		pszDirectory[(wcslen(pszDirectory)-1)]	= '\0';
	}

	if	( pszDirectory[0] == '\\' )
			pszDirectory++;

	if	( !(pszPathRoot[(wcslen(pszPathRoot)-1)] == '\\') )
	{
		wcscat(pszPathRoot, L"\\");
	} 

	swprintf(pszSharePath, L"%s%s", pszPathRoot, pszDirectory);
	swprintf(pszShareName, pszDirectory);

	//MessageBox(NULL, pszSharePath, pszShareName,  MB_OK);
 


	//we suppoze the AccountName was created before and added to local administrators group
	//if not, return an error

	//get security descriptor to AccountName
	if ( !GetAccess(AccountName, sd, szError)	)							return false;

	//new share
	if ( !ShareCreate(pszSharePath, pszShareName, sd, szError) )			return false;



	return true;
}

//*********************************************************************************************
//DeleteShare
//delete of share
//
//*********************************************************************************************
bool DeleteShare(LPTSTR  pszPathRoot, LPTSTR pszDirectory, LPTSTR &szError)
{
	NET_API_STATUS nas;

	nas	=	NetShareDel(NULL, pszDirectory, 0);

	if(nas != NO_ERROR) 
	{
		if (nas != NERR_NetNameNotFound)		//no error if not find the share
		{
			TCHAR		szMsgBuf[256];
			
			switch (nas)
			{
			case	ERROR_ACCESS_DENIED:	wcscpy(szMsgBuf, L"The user does not have access to the requested information.");break;
			case	ERROR_INVALID_PARAMETER:wcscpy(szMsgBuf, L"The specified parameter is invalid.");break;
			case	ERROR_NOT_ENOUGH_MEMORY:wcscpy(szMsgBuf, L"Insufficient memory is available.");break;
			case	NERR_NetNameNotFound:	wcscpy(szMsgBuf, L"TThe share name does not exist.");break;
				
			default:						wcscpy(szMsgBuf, L"Unspecified error.");break;
			}
			
			// Display the string.
			swprintf(szError, L"Error in NetShareDel:  %s", szMsgBuf);
			
			return false;
		}
	}

	return true;
}

#endif (__SHAREMANNAGER_H__)
