// Server.h : Declaration of the CServer

#ifndef __SERVER_H_
#define __SERVER_H_

#include "resource.h"       // main symbols
#include "ReplicationServerCP.h"

/////////////////////////////////////////////////////////////////////////////
// Helper Macros

#define CHECKHR(hr, desc) { if (FAILED(hr)) { Error(desc,0,0,GUID_NULL,hr); return E_FAIL; } }
#define CHECKHR_UNLOCK(hr, desc) { if (FAILED(hr)) { Error(desc,0,0,GUID_NULL,hr); Unlock(); return E_FAIL; } }
#define CHECKRESULTS_UNLOCK(hr, success, desc) { if (FAILED(hr) || !success) { Error(desc,0,0,GUID_NULL,hr); Unlock(); return E_FAIL; } }


#define RETURN_ERROR(desc) { hr = E_FAIL; 	hr = Error(desc, IID_IServer, hr); 	return hr; 	}
#define RETURN_ERROR_INSIDE(desc) { *bstrOutError	= desc; 	return false; 	}


#define MACROBEGINTRY _bstr_t			bstrErrDescription(L"");		\
	HRESULT			hr = S_OK;											\
	try																	\
	{	
#define MACROENDTRY }													\
	catch(_com_error err)												\
	{																	\
		bstrErrDescription = err.Description();							\
		hr = err.Error();												\
	}																	\
	catch(...)															\
	{																	\
	bstrErrDescription = L"Some unnatended Error";						\
		hr = E_FAIL;													\
	}																	\
	if(FAILED(hr))														\
		hr = Error((LPOLESTR)bstrErrDescription, IID_IServer, hr);	\
	return hr;		

#define MACROBEGINTRY_INSIDE _bstr_t			bstrErrDescription(L"");\
	HRESULT			hr = S_OK;											\
	try																	\
	{	
#define MACROENDTRY_INSIDE }											\
	catch(_com_error err)												\
	{																	\
		bstrErrDescription = err.Description();							\
		hr = err.Error();												\
	}																	\
	catch(...)															\
	{																	\
		bstrErrDescription = L"Some unnatended Error";					\
		hr = E_FAIL;													\
	}																	\
	*bstrOutError	= bstrErrDescription;								\
	if(FAILED(hr))														\
		return false;													\
	else																\
		return true;

/////////////////////////////////////////////////////////////////////////////
/*
struct SCarry
	{
		LPVOID		pClass;
		BSTR		bstrName;
	};
*/
/////////////////////////////////////////////////////////////////////////////
#include <algorithm>
#include <vector>
using namespace std;

typedef vector<_bstr_t> NAMEVECTOR; 

// CServer
class ATL_NO_VTABLE CServer : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CServer, &CLSID_Server>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<CServer>,
	public IDispatchImpl<IServer, &IID_IServer, &LIBID_REPLICATIONSERVERLib>,
	public CProxyIServerEvents< CServer >
{
public:
	CServer()
	{
		InitializeCriticalSection(&m_cs);

		bServiceRunning		= false;
		m_bShareFirstTime	= true;
	} 

DECLARE_REGISTRY_RESOURCEID(IDR_SERVER)

DECLARE_PROTECT_FINAL_CONSTRUCT()

DECLARE_CLASSFACTORY_SINGLETON(CServer);

BEGIN_COM_MAP(CServer)
	COM_INTERFACE_ENTRY(IServer)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()

BEGIN_CONNECTION_POINT_MAP(CServer)
	CONNECTION_POINT_ENTRY(DIID_IServerEvents)
END_CONNECTION_POINT_MAP()


// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

public:

// IServer
public:
	STDMETHOD(PutClientName)(/*[in]*/ BSTR bstrIn);
	STDMETHOD(GetActiveClients)(/*[out, retval]*/ BSTR *pbstrOut);
	STDMETHOD(Syncronize)();
	STDMETHOD(Talk)(/*[out, retval]*/ long *pVal);
	STDMETHOD(PutResource)(/*[in]*/ BSTR bstrPathRoot, /*[in]*/ BSTR bstrDirectory, /*[in]*/ BSTR bstrResourceName);


private:
	void							MyUnlock();
	void							MyLock();

	bool							ReadSignals();

	CRITICAL_SECTION				m_cs;

	bool							bServiceRunning;
	bool							m_bShareFirstTime;

	NAMEVECTOR						m_NameVector;
};

#endif //__SERVER_H_
